
# q-expansion of newform 222.2.a.e, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 222
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [1], [0], [-1], [3], [-1], [-3], [-7], [3], [0], [2], [1], [-6], [-4], [6], [9], [0], [-10], [2], [12], [5], [2], [3], [-3], [2], [6], [14], [9], [5], [-6], [5], [18], [-12], [14], [-18], [5], [8], [11], [-15], [-9], [12], [20], [-9], [26], [-21], [-28], [-28], [-16], [-24], [-16], [30], [-24], [-28], [18], [-27], [6], [15], [8], [17], [9], [-1], [9], [-16], [0], [-10], [30], [-4], [5], [6], [26], [-6], [30], [-25], [20], [-16], [21], [-6], [8], [-3], [-22], [-15], [-10], [-39], [-25], [-10], [36], [-18], [8], [-6], [-34], [6], [-39], [38], [15], [-13], [-24], [21], [18], [8], [11], [-13], [30], [36], [-21], [-4], [38], [0], [24], [-6], [-25], [14], [2], [-24], [38], [20], [-12], [5], [3], [24], [24], [-25], [-19], [27], [18], [-40], [-24], [-25], [24], [8], [-34], [38], [-36], [32], [5], [54], [-16], [-21], [-34], [12], [-27], [-52], [45], [17], [-18], [17], [-12], [-37], [-21], [-13], [-18], [-58], [0], [-25], [18], [23], [24], [38], [18], [38], [-6], [6], [-24], [44], [-12], [-3], [-18], [-16], [53]]
